import {Response, NextFunction} from 'express';
import {environment} from './environment';

export function formatIp(ip: string) {
	if(!ip) return null;
	const ipv4 = ip.split(':').splice(-1)[0];
	if(ipv4 == '1') return '127.0.0.1';
	return ipv4;
}

export function loggerMiddleware(req: any, resp: Response, next: NextFunction) {
	req.start = new Date().getTime();
	req.ipV4 = formatIp(req.ip);

	if(req.path == '/api/healthcheck' && environment.production)
		return next();

	console.debug(`[${req.ipV4}] ${req.method} ${decodeURI(req.url)}`);
	resp.on('finish', () => {
		if(resp['logged']) return;
		let units = 'ms', elapsed: any = new Date().getTime() - req.start;
		if(elapsed > 1000) {
			units = 's';
			elapsed = (elapsed / 1000).toFixed(1);
		}
		if(units == 's' && elapsed > 100) {
			units = 'm';
			elapsed = (elapsed / 60).toFixed(1);
		}
		const serverError = resp.statusCode >= 500 && resp.statusCode < 600;
		const clientError = resp.statusCode >= 400 && resp.statusCode < 500;
		console[serverError ? 'error' : (clientError && !environment.production) ? 'warn' : 'log'](`(${elapsed}${units}) [${req.ipV4}] ${req?.session?.user ? `(${req.session?.user?.username}) ` : ''}${req.method} ${decodeURI(req.url)} -> ${resp.statusCode}`);
	});

	next();
}
